#!/usr/bin/env python3

import os
import sys
import re
import datetime
import socket
import urllib.parse

emptypage = "https://www.ida.liu.se/edu/ugrad/datortenta/empty"
errorpage = "https://www.ida.liu.se/edu/ugrad/datortenta/blocked.php"

emptyrex = "^https?://www\.ida\.liu\.se/edu/ugrad/datortenta/empty"
errorrex = "^https?://www\.ida\.liu\.se/edu/ugrad/datortenta/blocked\.php\?.*"

filterlog = "/var/log/squid/filter.log"
basedir = "/usr/libexec/squid/helpers"
hostname = socket.gethostname()

def block_response(url):
    quoted_url = urllib.parse.quote(url)
    if re.match('.*\.s?html([#?].*)?', url) or re.match('.*/[^./]*', url):
        return 'OK status=307 url="{}?requested={}"'.format(errorpage, quoted_url)
    else:
        return 'OK status=307 url="{}"'.format(emptypage)

    # Note:
    #  With a redirector squid ask the client to fetch another page.
    #  With a re-writer squid ask the web server to fetch the new page.
    #  "status" can only be used in the "Using an HTTP redirector" case,
    #  not in the "Using a re-writer to mangle the URL as it passes" case.
    # See: 
    #  https://wiki.squid-cache.org/Features/Redirectors
    
def modify_url(line, ruleset):
### [channel-ID <SP>] URL <SP> client_ip "/" fqdn <SP> user <SP> method [<SP> kv-pairs]<NL>
    list = line.split(' ')

    if list[0].isdigit():
        url = list[1]
    else:
        url = list[0]

    for i in list[2:]:
        if i == "CONNECT":
            return "OK"
        
    if url == "https://www.ida.liu.se/~examadm/start.html":
        return "OK rewrite-url=\"https://www.ida.liu.se/edu/ugrad/datortenta/start.html"
        
    r = re.match('https://www\.ida\.liu\.se/~opendsa/(.*)', url)
    if r != None:
        return 'OK rewrite-url="http://{}:12000/{}"'.format(hostname, r.group(1))

    for rule in ruleset:
        pattern = 0
        allowed = 1
        if rule[pattern].fullmatch(url) != None:

            if rule[allowed]:
                return "OK"
            else:
                return block_response(url)

    return block_response(url)


def load_rules(ruleset, filename):
    with open(filename, 'r') as f:

        for line in f:
            if not re.match('(^#.*)|(^//.*)|(^;.*)|(^--.*)|^ *$', line):
                columns = line.strip().split()
                ruleset.append( [ re.compile(columns[0]), columns[1].lower() == 'true' ] )


def main():
    
    ruleset = list()

    block_all = basedir + "/../BLOCK_ALL"
    if os.path.isfile(block_all):
        ruleset.append( [re.compile(emptyrex), True] )
        ruleset.append( [re.compile(errorrex), True] )
        ruleset.append( [re.compile("^.*"), False] )

    devel_rules = basedir + "/../devel.rules"
    if 'devel' in hostname and os.path.isfile(devel_rules):
        load_rules(ruleset, devel_rules)

    # load opendsa first to let it override default rules
    load_rules(ruleset, basedir + "/rules.d/opendsa.rules")
    load_rules(ruleset, basedir + "/rules.d/default.rules")
    load_rules(ruleset, basedir + "/rules.d/rstudio.rules")
    load_rules(ruleset, basedir + "/rules.d/cplusplus.rules")
    load_rules(ruleset, basedir + "/rules.d/python.rules")
    load_rules(ruleset, basedir + "/rules.d/java.rules")
    load_rules(ruleset, basedir + "/rules.d/ruby.rules")
    load_rules(ruleset, basedir + "/rules.d/sas.rules")
    load_rules(ruleset, basedir + "/rules.d/translate.rules")

    with open(filterlog, 'a') as log:
        try:
            while True:
            
                line = sys.stdin.readline().strip()

                log.write('{}: {}\n'.format(datetime.datetime.now().strftime("%Y-%m-%d_%H-%M"), line))
                log.flush()

                log.write("From: " + line + "\n")
                log.flush()
            
                new_url = modify_url(line, ruleset)

                log.write("To: " + new_url + "\n")
                log.flush()

                sys.stdout.write(new_url + '\n')
                sys.stdout.flush()
        except Exception as e:
            sys.stderr.write( e )
            sys.stderr.flush()

main()
